import 'package:flutter/services.dart';
import 'package:flutter/widgets.dart';

class KvalifikaSdk extends StatefulWidget {
  static const MethodChannel _channel = const MethodChannel('kvalifika_sdk');

  final Function onInitialize;
  final Function(String sessionId) onStart;
  final Function(String sessionId) onFinish;
  final Function(KvalifikaSdkError error, String message) onError;

  final Widget Function(KvalifikaSdk sdk) builder;
  final String appId;
  final KvalifikaSdkLocale locale;
  final String logo;
  final String documentIcon;
  final String cancelIcon;
  final String activeFlashIcon;
  final String inactiveFlashIcon;

  KvalifikaSdk({
    @required this.builder,
    @required this.appId,
    @required this.onInitialize,
    @required this.onStart,
    @required this.onFinish,
    @required this.onError,
    this.locale = KvalifikaSdkLocale.EN,
    this.logo,
    this.documentIcon,
    this.cancelIcon,
    this.activeFlashIcon,
    this.inactiveFlashIcon,
  });

  void startSession() async {
    await _channel.invokeMethod('sdkStartSession');
  }

  @override
  _KvalifikaSdkState createState() {
    _channel.setMethodCallHandler((call) {
      if (call.method == "sdkOnInitialize") {
        this.onInitialize();
      } else if (call.method == "sdkOnStart") {
        this.onStart(call.arguments);
      } else if (call.method == "sdkOnFinish") {
        this.onFinish(call.arguments);
      } else if (call.method == "sdkOnError") {
        List args = call.arguments.split("___");
        String errorName = args[0];
        String message = args[1];

        switch (errorName) {
          case 'INVALID_APP_ID':
            this.onError(KvalifikaSdkError.INVALID_APP_ID, message);
            break;
          case 'USER_CANCELLED':
            this.onError(KvalifikaSdkError.USER_CANCELLED, message);
            break;
          case 'TIMEOUT':
            this.onError(KvalifikaSdkError.TIMEOUT, message);
            break;
          case 'SESSION_UNSUCCESSFUL':
            this.onError(KvalifikaSdkError.SESSION_UNSUCCESSFUL, message);
            break;
          case 'ID_UNSUCCESSFUL':
            this.onError(KvalifikaSdkError.ID_UNSUCCESSFUL, message);
            break;
          case 'CAMERA_PERMISSION_DENIED':
            this.onError(KvalifikaSdkError.CAMERA_PERMISSION_DENIED, message);
            break;
          case 'LANDSCAPE_MODE_NOT_ALLOWED':
            this.onError(KvalifikaSdkError.LANDSCAPE_MODE_NOT_ALLOWED, message);
            break;
          case 'REVERSE_PORTRAIT_NOT_ALLOWED':
            this.onError(
                KvalifikaSdkError.REVERSE_PORTRAIT_NOT_ALLOWED, message);
            break;
          case 'FACE_IMAGES_UPLOAD_FAILED':
            this.onError(KvalifikaSdkError.FACE_IMAGES_UPLOAD_FAILED, message);
            break;
          case 'DOCUMENT_IMAGES_UPLOAD_FAILED':
            this.onError(
                KvalifikaSdkError.DOCUMENT_IMAGES_UPLOAD_FAILED, message);
            break;
          case 'COMPARE_IMAGES_FAILED':
            this.onError(KvalifikaSdkError.COMPARE_IMAGES_FAILED, message);
            break;
          case 'UNKNOWN_INTERNAL_ERROR':
            this.onError(KvalifikaSdkError.UNKNOWN_INTERNAL_ERROR, message);
            break;
          default:
            this.onError(KvalifikaSdkError.UNKNOWN_INTERNAL_ERROR, message);
            break;
        }
      }
      return;
    });

    _channel.invokeMethod('sdkInitialize', {
      'appId': appId,
      'locale': locale.toString().split('.')[1],
      'logo': logo,
      'documentIcon': documentIcon,
      'cancelIcon': cancelIcon,
      'activeFlashIcon': activeFlashIcon,
      'inactiveFlashIcon': inactiveFlashIcon,
    });

    return _KvalifikaSdkState();
  }
}

class _KvalifikaSdkState extends State<KvalifikaSdk> {
  @override
  Widget build(BuildContext context) {
    return this.widget.builder(this.widget);
  }
}

enum KvalifikaSdkError {
  INVALID_APP_ID,
  USER_CANCELLED,
  TIMEOUT,
  SESSION_UNSUCCESSFUL,
  ID_UNSUCCESSFUL,
  CAMERA_PERMISSION_DENIED,
  LANDSCAPE_MODE_NOT_ALLOWED,
  REVERSE_PORTRAIT_NOT_ALLOWED,
  FACE_IMAGES_UPLOAD_FAILED,
  DOCUMENT_IMAGES_UPLOAD_FAILED,
  COMPARE_IMAGES_FAILED,
  UNKNOWN_INTERNAL_ERROR,
}

enum KvalifikaSdkLocale {
  EN,
  GE,
}
